/*
* file CcView.java
*
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
* 
* com.ibm.rational.wvcm.stp.cc.CcView
*
* (C) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
* Note to U.S. Government Users Restricted Rights:  Use, duplication or  
* disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
*/

package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.io.File;
import java.util.Map;

import javax.wvcm.Feedback;
import javax.wvcm.Resource;
import javax.wvcm.Stream;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.StpActivity;
import com.ibm.rational.wvcm.stp.cc.CcFileAreaLockedCallback.CcFileAreaLockInfo;
import com.ibm.rational.wvcm.stpex.StpExEnumeration;

/**
 * <p>A proxy for a ClearCase view.  ClearCase "view" and WVCM "workspace"
 * are equivalent terms for the same type of resource. 
 * </p>
 * <p>In this release, only ClearCase CCRC web views are supported.
 * ClearCase dynamic and snapshot views are not currently supported.
 * </p>
 */
public interface CcView 
    extends CcDirectory, javax.wvcm.Workspace
{
    /** Values for view text mode */
    enum TextMode implements StpExEnumeration {
        
        /**
         * A transparent interop text mode. The line terminator for text files 
         * is a single <code>NL</code> character. The view does not transform 
         * text file line terminators in any way. This is the default text mode 
         * if no value is set upon view creation.
         */
        TRANSPARENT("transparent"),
        
        /**
         * An insert_cr interop text mode. The view converts <code>NL</code> line 
         * terminators to the <code>CR NL</code> sequence when reading from a VOB,
         * and <code>CR NL</code> line terminators to single <code>NL</code>
         * characters when writing to the VOB.
         */
        INSERT_CR("insert-cr"),
        
        /**
         * An nl_to_cr interop text mode. The view converts <code>NL</code> line
         * terminators to <code>CR</code> characters when reading from a VOB, and
         * <code>CR</code> line terminators to single <code>NL</code> characters
         * wehn writing to the VOB.
         */
        NL_TO_CR("nl-to-cr"),
        
        /**
         * A strip_cr interop text mode. The view converts <code>CR NL</code> line 
         * terminators to <code>NL</code> when reading from a VOB, and <code>NL</code> 
         * line terminators back to the <code>CR NL</code> sequence when writing to the VOB.
         */
        STRIP_CR("strip-cr");
        
        private String m_name;

        private TextMode(String name) { m_name = name; }

        /* (non-Javadoc)
         * @see java.lang.Object#toString()
         */
        public String toString() { return m_name; }
    }
    
    /** Flags for the <code>doSynchronizeFileAreaDb</code> method */
    enum SynchronizeFileAreaDbFlag implements StpExEnumeration {
        
        /**
         * Synchronize the view's client-side vob database with the
         * correct information from the server. Specifying this flag
         * requires a server connection.
         */
        FILE_AREA_VOB_DB("file-area-vob-db"),
        
        /**
         * Synchronize the view's client-side modified files database
         * so that it correctly reflects files that have been modified
         * as a result of checkouts or hijacks. Specifying this flag
         * does <i>not</i> require a server connection.
         */
        FILE_AREA_MODIFIED_FILES_DB("file-area-modified-files-db");
    
        private String m_name;

        private SynchronizeFileAreaDbFlag(String name) { m_name = name; }

        /* (non-Javadoc)
         * @see java.lang.Object#toString()
         */
        public String toString() { return m_name; }
    }
    
    /**
     *<p>
     * Binds a CcActivity proxy to a CqRecord proxy without making the
     * activity the current one for this view.
     * </p>
     * <p>
     * NOTE: Unlike most API methods, the optional property request will be executed
     * on the returned StpActivity, not the CcView proxy on which the method
     * was invoked.
     * </p>
     * 
     * <code>act</code> can either be a CcActivity proxy or a CqRecord proxy.
     * <p>
     * If <code>act</code> is a CcActivity proxy and the associated project is
     * not CQ enabled, this operation simply executes the provided property
     * request -- if any.  If the project is CQ enabled, it additionally validates
     * that the bound CQ record is in an active state.
     * </p>
     * <p>
     * If <code>act</code> is a CqRecord proxy, this operation is more
     * involved.  First, <code>act</code> is automatically transitioned to an
     * active state.  If this transition involves required fields, the caller
     * may be prompted to provide values for those fields.  Next a CcActivity 
     * resource is created in this view's stream and is bound to
     * <code>act</code>.  
     * </p>
     * @param act the activity to work on - either a CqRecord or CcActivity
     * @param feedback optional property request
     * @return new proxy for the StpActivity, with requested properties
     * @see javax.wvcm.Workspace#CURRENT_ACTIVITY
     * @see com.ibm.rational.wvcm.stp.StpActivity#BOUND_CC_ACTIVITY
     * @see com.ibm.rational.wvcm.stp.StpActivity#BOUND_CQ_RECORD
     */
    public StpActivity doBindActivity(StpActivity act, Feedback feedback) throws WvcmException;

    /**
     * <p>Create a new ClearCase web view based on this CcView proxy.
     * </p>
     * <p>
     * Preconditions:
     * </p>
     * <ul>
     * <li>This proxy must have a file-based location representing the desired
     * path of the root directory of the file area on the local machine.
     * The directory must not yet exist.</li>
     * <li>The location's leaf name (the directory name) will be used as
     * the view tag of the new view.  A view having that view-tag must not
     * already exist.</li>
     * <li>To create a view associated with a UCM stream, specify that stream
     * by setting this view proxy's STREAM property.</li>
     * </ul>
     * <p>
     * Postconditions:
     * </p>
     * <ul>
     * <li>A ClearCase web view is created, with the view tag
     * and local file area as described above.</li>
     * <li>If the view proxy's STREAM property was set, the view will be
     * associated with that stream.</li>
     * <li>If the view proxy's VIEW_TAG_STRING property was set, the view will
     * be assigned that tag.  Otherwise, the file area's root directory name
     * will be used.</li>
     * </ul>
     * @throws WvcmException if the preconditions are not met, or if there is an
     * error creating the view. 
     */
    public CcView doCreateCcWebView(Feedback feedback) throws WvcmException;

    /**
     * <p>Upgrade this web view's file area.</p>
     * <p>Preconditions:</p>
     * <ul>
     * <li>This web view's file area schema version must be older than the
     * version supported by the currently running CM API file area management
     * code.
     * </li>
     * </ul>
     * <p>Postconditions:</p>
     * <ul>
     * <li>This web view's file area schema will be upgraded to the currently
     * supported version.</li>
     * </ul>
     * <p>This operation has no effect on non-web views, and on web views that
     * are already compatible.
     * </p>
     * @throws WvcmException if the preconditions are not met, or if there is an
     * error upgrading the file area. 
     */
    public CcView doUpgradeFileArea(Feedback feedback) throws WvcmException;

    /**
     * Work on the specified activity in this CC view.
     * <code>act</code> can either be a CcActivity proxy or a CqRecord proxy.
     * <p>
     * If <code>act</code> is a CcActivity proxy and the associated project is
     * not CQ enabled, this operation simply makes it the current activity 
     * in this view.  If the project is CQ enabled, it additionally validates 
     * that the bound CQ record is in an active state.
     * </p>
     * <p>
     * If <code>act</code> is a CqRecord proxy, this operation is more
     * involved.  First, <code>act</code> is automatically transitioned to an
     * active state.  If this transition involves required fields, the caller
     * may be prompted to provide values for those fields.  Next a CcActivity 
     * resource is created in this view's stream and is bound to
     * <code>act</code>.  Finally, it makes the new CC activity resource the
     * current activity in this view.
     * </p>
     * @param act the activity to work on - either a CqRecord or CcActivity
     * @param feedback optional property request
     * @return new proxy for this view, with requested properties
     * @see javax.wvcm.Workspace#CURRENT_ACTIVITY
     * @see com.ibm.rational.wvcm.stp.StpActivity#BOUND_CC_ACTIVITY
     * @see com.ibm.rational.wvcm.stp.StpActivity#BOUND_CQ_RECORD
     */
    public CcView doWorkOnActivity(StpActivity act, Feedback feedback) throws WvcmException;

    /**
     * <p>
     * Transitions the specified activity to the default completed state.
     * <code>act</code> can either be a CcActivity proxy or a CqRecord proxy.
     * </p>
     * <p>
     * NOTE: Unlike most API methods, the optional property request will be executed
     * on the returned StpActivity, not the CcView proxy on which the method
     * was invoked.
     * </p>
     * <p>Preconditions:</p>
     * <ul>
     * <li><code>act</code> is a bound CqRecord/CcActivity pair in a CQ-enabled
     * context.</li>
     * </ul>
     * <p>Postconditions:</p>
     * <ul>
     * <li>If <code>act</code>'s CcActivity has checkouts, the operation is
     * cancelled and there is no change to the activity.</li>
     * <li>Transitions <code>act</code>'s CqRecord to the default completed state. 
     * If this transition involves required fields, the caller may be prompted 
     * to provide values for those fields.</li>
     * <li>Unsets <code>act</code>'s CcActivity from all views'
     * {@link javax.wvcm.Workspace#CURRENT_ACTIVITY} property.</li>
     * </ul>
     * @param act the activity to finish
     * @param feedback optional property request for the activity
     * @return new proxy for the activity, with requested properties
     * @throws WvcmException if the preconditions are not met.
     */
    public StpActivity doFinishActivity(StpActivity act, Feedback feedback) 
        throws WvcmException;

    /**
     * <p>
     * Synchronize this file area's local databases to accurately reflect the 
     * current state of the file area. The databases available to synchronize are:
     * </p>
     * <ul>
     * <li>VOB database: This contains the loaded vobs in the view. A server connection
     * is required to synchronize this database.</li>
     * <li>Modified files database: This contains the checkouts and hijacks in the view.
     * A server connection <i>not</i> required to synchronize this database.</li>
     * </ul>
     * @param flags array of flags which specify the databases to synchronize.
     * @param feedback optional property request for the view
     * @return A new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException
     */
    public CcView doSynchronizeFileAreaDb(SynchronizeFileAreaDbFlag[] flags, Feedback feedback) 
        throws WvcmException;

    /**
     * Register this local web view's file area in the local file area registry.
     * Web views are registered automatically when created, but may need to be
     * re-registered if the registry is accidentally deleted, etc.
     */
    public void registerFileArea() throws WvcmException;

    /**
     * Remove this local web view's file area from the local file area registry.
     * Once removed, this view will no longer show up when listing local views.
     * @see com.ibm.rational.wvcm.stp.cc.CcProvider#getClientViewList(javax.wvcm.PropertyRequestItem.PropertyRequest)
     */
    public void unregisterFileArea() throws WvcmException;

    /**
     * If this is a local web view, the URL of the CM Server where this
     * view's view database resides.
     */
    PropertyName<String> SERVER_URL =
        new PropertyName<String>(PROPERTY_NAMESPACE, "server-url");
   
    /**
     * Get the value of this proxy's {@link #SERVER_URL} property.
     * @return this view's server URL
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    public String getServerUrl() throws WvcmException;

    /**
     * Change the URL used to connect to the CM server on which this web view resides.
     * Note: Web views cannot be moved from one CM server to another.
     * Use this method only to change the server URL, not to try to connect to
     * a different server.
     * <p>
     * This may be necessary if, for example:
     * <ul>
     * <li>The CM server administrator changes the CM server's port number</li>
     * <li>The client needs a secure HTTP connection to the CM server:
     *     "https://..." instead of "http://..."</li>
     * <li>The CM server is moved to a different internet subdomain</li>
     * </ul>
     * @param updatedUrl the updated URL of this web view's CM server
     * @throws WvcmException
     */
    public void updateServerUrl(String updatedUrl) throws WvcmException;

    /**
     * This view's view tag as a string.
     */
    PropertyName<String> VIEW_TAG_STRING =
        new PropertyName<String>(PROPERTY_NAMESPACE, "view-tag-string");
  
    /**
     * Get the value of this proxy's {@link #VIEW_TAG_STRING} property.
     * @return this view's view tag
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    public String getViewTagString() throws WvcmException;

    /**
     * Set the value of this view's {@link #VIEW_TAG_STRING} property.
     * This property may only be set at view creation time.
     * @param viewTag the view tag for the new view
     */
    void setViewTagString(String viewTag);
   
    /**
     * This view's view tag as a {@link CcViewTag} resource.
     */
    PropertyName<CcViewTag> VIEW_TAG =
        new PropertyName<CcViewTag>(PROPERTY_NAMESPACE, "view-tag");
   
    /**
     * Get the value of this proxy's {@link #VIEW_TAG} property.
     * @return this view's view tag
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    public CcViewTag getViewTag() throws WvcmException;
   
    /**
     * Whereas a CcView resource's {@link javax.wvcm.Folder#CHILD_MAP} property
     * returns the root directories of <i>all</i> VOBs, LOADED_CHILD_MAP only
     * returns the root directories of VOBs that are partially or fully loaded
     * in this view. 
     */
    PropertyName<Map<String,Resource>> LOADED_CHILD_MAP =
        new PropertyName<Map<String,Resource>>(PROPERTY_NAMESPACE, "loaded-child-map");
   
    /**
     * Get the value of this proxy's {@link #LOADED_CHILD_MAP} property.
     * @return this view's loaded child map
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    public Map<String,Resource> getLoadedChildMap() throws WvcmException;

    /**
     * This view's file area root directory on the local machine.
     * The value of this property will be null if this is not a web view,
     * or if it does not have a file area on the local machine.
     */
    PropertyName<File> FILE_AREA_ROOT_DIRECTORY =
        new PropertyName<File>(PROPERTY_NAMESPACE, "file-area-root-directory");
   
    /**
     * Returns the value of the {@link #FILE_AREA_ROOT_DIRECTORY} property.
     * @return This view's copy area root directory, or null if it has
     *         no copy area on the local machine
     * @throws WvcmException if this property is not defined by this proxy.
     */
    public File getFileAreaRootDirectory() throws WvcmException;

    /**
     * Is this view's local file area schema version older than the version
     * supported by the running CM API file management code? If so, the file
     * area needs to be upgraded before it can be used.
     * 
     * @see com.ibm.rational.wvcm.stp.cc.CcView#doUpgradeFileArea(Feedback)
     */
    PropertyName<Boolean> FILE_AREA_NEEDS_UPGRADE =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "file-area-needs-upgrade");
   
    /**
     * Get the value of this view's {@link #FILE_AREA_NEEDS_UPGRADE} property.
     * @return true if this view's local file area needs upgrading; false otherwise.
     * @throws WvcmException if this property is not defined by this proxy.
     */
    boolean getFileAreaNeedsUpgrade() throws WvcmException;
   
    /**
     * Is this view associated with a UCM stream?.
     */
    PropertyName<Boolean> IS_UCM_VIEW =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-ucm-view");
   
    /**
     * Get the value of this view's {@link #IS_UCM_VIEW} property.
     * @return true if this view represents a UCM view; false otherwise.
     * @throws WvcmException if this property is not defined by this proxy.
     */
    boolean getIsUcmView() throws WvcmException;
   
    /**
     * This view's config spec.
     */
    PropertyName<CcConfigSpec> CONFIG_SPEC =
        new PropertyName<CcConfigSpec>(PROPERTY_NAMESPACE, "config-spec");
    
    /**
     * Get the value of this view's {@link #CONFIG_SPEC} property.
     * @return this view's config spec as a CcConfigSpec instance
     */
    CcConfigSpec getConfigSpec() throws WvcmException;

    /**
     * Set the value of this view's {@link #CONFIG_SPEC} property.
     * @param configSpec the new config spec for this view
     */
    void setConfigSpec(CcConfigSpec configSpec);
    
    /**
     * The text mode of the view. The text mode specifies the line
     * terminator sequence for text files in the view. If no value is
     * set upon view creation, defaults to {@link TextMode#TRANSPARENT}.
     */
    PropertyName<TextMode> TEXT_MODE =
        new PropertyName<TextMode>(PROPERTY_NAMESPACE, "text-mode");
    
    /**
     * Get the value of this view's {@link #TEXT_MODE} property.
     * @return enumeration value representing this view's text mode.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    TextMode getTextMode() throws WvcmException;
    
    /**
     * Set the value of this view's {@link #TEXT_MODE} property. This
     * property may only be set at view creation time.
     * @param textMode the text mode of the view.
     */
    void setTextMode(TextMode textMode);
    
    /**
     * Break a file area lock on this view with the given lock info.
     * @param lockInfo information about the lock
     * @return true if the lock was broken; false otherwise
     * @throws WvcmException if a problem occurred breaking the lock
     */
    boolean breakFileAreaLock(CcFileAreaLockInfo lockInfo) throws WvcmException;
    
    /**
     * CcViewAccessInfo object contains the supported view access properties.
     */
    public static final PropertyName<CcViewAccessInfo> VIEW_ACCESS_INFO =
        new PropertyName<CcViewAccessInfo>(PROPERTY_NAMESPACE, "view-access-info");

    /**
     * Returns the value of this proxy's {@link #VIEW_ACCESS_INFO} property.
     * 
     * @return the CcViewAccessInfo object.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    public CcViewAccessInfo getViewAccessInfo() throws WvcmException;

    /**
     * <p>
     * The permissions applied to this resource.
     * </p>
     */
    public static final PropertyName<CcPermissions> PERMISSIONS =
        new PropertyName<CcPermissions>(PROPERTY_NAMESPACE, "cc-permissions");
    
    /**
     * Returns the value of this proxy's {@link #PERMISSIONS} property.
     * 
     * @return A permissions object from which specific permissions 
     * 	       information can be extracted.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    public CcPermissions getPermissions() throws WvcmException;

    /**
     * Does this view have non-shareable derived objects?
     */
    public static final PropertyName<Boolean> IS_EXPRESS =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-express");
    
    /**
     * Returns the value of this proxy's {@link #IS_EXPRESS} property.
     * 
     * @return true if this view has non-shareable DOs; false otherwise.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    public Boolean getIsExpress() throws WvcmException;

    /**
     * Are this view's permissions valid?
     */
    public static final PropertyName<Boolean> ARE_PERMISSIONS_VALID =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "are-permissions-valid");
    
    /**
     * Returns the value of this proxy's {@link #ARE_PERMISSIONS_VALID} property.
     * 
     * @return true if this view's permissions are valid; false otherwise.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    public Boolean getArePermissionsValid() throws WvcmException;

    /**
     * Is this view read-only?
     */
    public static final PropertyName<Boolean> IS_READ_ONLY =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-read-only");
    
    /**
     * Returns the value of this proxy's {@link #IS_READ_ONLY} property.
     * 
     * @return true if this view is read-only; false otherwise.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    public Boolean getIsReadOnly() throws WvcmException;
    
     /**
     * Set the {@link #STREAM} property.
     * 
     * @param stream the {@link Stream} object that
     * identifies the {@link #STREAM} for this Workspace.
     * @see #getStream
     */
    public void setStream(Stream stream);
}
